local core = {}
local db = nil
_G.UnconversableGnome = core

local cmd = {}
function core:Register(command, func)
	if type(command) ~= "string" then return end
	local c = command:lower()
	if c:find("%s") then error("Commands can't contain spaces, I'm not a mind reader!") end
	if cmd[c] then error("Command " .. c .. " already registered, get a clue!") end
	cmd[c] = func
end
function core:IsRegistered(command)
	return cmd[command:lower()] and true or false
end

local addon = CreateFrame("Frame")
addon:SetScript("OnEvent", function(self, event, ...) self[event](self, event, ...) end)

do
	local defaults = {
		definitions = {},
	}
	function addon:ADDON_LOADED(event, addon)
		if addon:lower() ~= "unconversablegnome" then return end
		print("loading")
		if not UnconversableGnomeDB then UnconversableGnomeDB = {} end
		local realm = GetRealmName()
		if not UnconversableGnomeDB[realm] then UnconversableGnomeDB[realm] = {} end
		local faction = UnitFactionGroup("player")
		if not UnconversableGnomeDB[realm][faction] then UnconversableGnomeDB[realm][faction] = {} end
		db = UnconversableGnomeDB[realm][faction]
		for k, v in pairs(defaults) do
			if not db[k] then db[k] = v end
		end
	end
end
addon:RegisterEvent("ADDON_LOADED")

local eventToChannel = {
	CHAT_MSG_GUILD = "GUILD",
	CHAT_MSG_PARTY = "PARTY",
	CHAT_MSG_RAID = "RAID",
	CHAT_MSG_RAID_LEADER = "RAID",
	CHAT_MSG_OFFICER = "OFFICER",
	--CHAT_MSG_CHANNEL = "CHANNEL",
	CHAT_MSG_WHISPER = "WHISPER",
}
local function parse(input, a)
	local raw = input:match("^~(.*)")
	if not raw then return end
	local c, i = nil, nil
	if raw:find("%s") then
		c, i = raw:match("(.-)%s(.*)")
	else
		c = raw
	end
	if not c then return end
	if cmd[c] then
		local r = cmd[c](i, a)
		if type(r) ~= "string" then error("Command " .. tostring(c) .. " didn't return a string.") end
		return r
	elseif type(i) == "string" and i:find("^is%s") then
		local definition = i:match("^is%s(.*)")
		db.definitions[c] = definition
		return "Alright, " .. a .. "."
	elseif db.definitions[c] then
		local add = type(i) == "string" and i or a
		return db.definitions[c]:format(add)
	end
end
local function printout(event, author, channelNumber, ...)
	local channel = eventToChannel[event]
	if not channel then return end
	for i = 1, select("#", ...) do
		local text = select(i, ...)
		if channel == "WHISPER" then
			SendChatMessage(text, channel, nil, author)
		elseif channel == "CHANNEL" then
			SendChatMessage(text, channel, nil, channelNumber)
		else
			SendChatMessage(text, channel)
		end
	end
end
local function incoming(self, event, msg, author, language, _, _, _, _, channelNumber)
	local ret = parse(msg, author)
	if type(ret) ~= "string" then return end
	printout(event, author, channelNumber, strsplit("\n", ret))
end
for k in pairs(eventToChannel) do
	addon[k] = incoming
	addon:RegisterEvent(k)
end

--------------------------------------------------------------------------------
-- "Core" commands .. lol
--

-- Make people able to spawn the bot in other channels than the one they
-- invoke the command in, by wrapping the command in guild|raid|party|officer,
-- like whispering this to the bot owner will output it in party chat:
-- ~party seen Someone
do
	local function wrap(input, a) return parse("~" .. input, a) end
	core:Register("guild", function(...) SendChatMessage(wrap(...), "GUILD") end)
	core:Register("party", function(...) SendChatMessage(wrap(...), "PARTY") end)
	core:Register("raid", function(...) SendChatMessage(wrap(...), "RAID") end)
	core:Register("officer", function(...) SendChatMessage(wrap(...), "OFFICER") end)
end
do
	core:Register("ping", function()
		return "pong!"
	end)
end

do
	local t = {
		"Don't push it!",
		"You've really done it now.",
		"I'll get you!",
		"Grrrr...",
		"Please. Just stop.",
		"For the love of Ragnaros!",
		"All good things must come to an end.",
		"I'm dead now. Thanks a lot.",
	}
	local function f()
		local r = math.random(1, #t)
		return t[r]
	end
	for i = 1, 10 do
		core:Register(("~"):rep(i), f)
	end
end


--------------------------------------------------------------------------------
-- Example commands, should be implemented in their own files and more thoroughly,
-- I just made them work somewhat to test things.
--

do
	local insults = {
		"You're such a nitwit, %s!",
		"When %s goes shopping, the shop shops him instead.", -- lolwut
		"That's enough, %s. Noone cares.",
		"Please continue, %s. I enjoy ignoring you.",
		"Your mom is coming soon, %s, don't worry.",
		"So is your face, %s.",
		"I never forget a face, %s. But in your case I'll make an exception.",
		"Your mother is so ugly, %s, that she made an onion cry.",
		"I don't know what makes you so dumb, %s, but it really works!",
		"Brains aren't everything. In fact, in the case of %s, they're nothing.",
		"I'm impressed with %s. I've never met such a small mind inside such a big head.",
		"What do you want to do if you grow up, %s?",
		"You're not yourself today, %s. I noticed the improvement immediately.",
	}
	core:Register("lart", function(input, author)
		if not input then return end
		local r = math.random(1, #insults)
		return insults[r]:format(input)
	end)
end

do
	local lastSeen = {}
	local updating = nil
	local f = CreateFrame("Frame")
	f:SetScript("OnEvent", function(self, event)
		if updating then return end
		updating = true
		--[[self:UnregisterEvent("GUILD_ROSTER_UPDATE")
		local offline = GetGuildRosterShowOffline()
		local selection = GetGuildRosterSelection()
		SetGuildRosterShowOffline(true)]]
		local numGuildMembers = GetNumGuildMembers()
		for i = 1, numGuildMembers do
			local n = GetGuildRosterInfo(i)
			if n then
				-- ninjaed from wowwiki
				local y, m, d, h = GetGuildRosterLastOnline(i)
				y,m,d,h = y and y or 0, m and m or 0, d and d or 0, h and h or 0
				local seen = (((y * 12) + m) * 30.5 + d) * 24 + h
				lastSeen[n:lower()] = seen
			end
		end
		--SetGuildRosterShowOffline(offline)
		--SetGuildRosterSelection(selection)
		updating = nil
		--self:RegisterEvent("GUILD_ROSTER_UPDATE")
	end)
	f:RegisterEvent("GUILD_ROSTER_UPDATE")
	local same = "You're right here, stupid!"
	local seen = "%s was last seen %d hours ago."
	local who = "I don't know who you're talking about, %s."
	local here = "%s is right here! Are you blind or what?"
	core:Register("seen", function(input, author)
		local i = input:lower()
		if i == author:lower() then
			return same
		elseif lastSeen[i] and lastSeen[i] > 0 then
			return seen:format(input, lastSeen[i])
		elseif lastSeen[i] then
			return here:format(input)
		else
			return who:format(author)
		end
	end)
end

